==> dddns-1.0.5.tar.gz: dddns-1.0.5/misc/routeadjust.sh <==
#! /bin/bash

# (c) 2004 Leading Edge Business Solutions and Andrew McGill
# Distribution permitted under terms of GPL licence, as contained
# in the file COPYING
#
VERSION='$Id: routeadjust.sh,v 1.9 2005/03/14 16:01:41 andrewm Exp $' 
VERSION="${VERSION/*,v }"
VERSION="${VERSION/ *}"
#
# Adjust routing for two ppp interfaces.
#
# We route via ppp0 for RSA and ppp1 for world. Additionally we
# route based on source address to avoid asymmetric situations.
# We assume iptables -t nat -A POSTROUTING -j MASQUERADE ... I
# think.  (If not, sorry about the misunderstanding.)
#
# Call this ...
#   * interface comes up - link from /etc/ppp/ip-up.d
#   * interface goes down - link from /etc/ppp/ip-down.d
#   * every so on - to check dddns
#

PATH=/bin:/sbin:/usr/sbin:/usr/local/sbin:/root/bin:/usr/local/bin:/usr/bin
export PATH

LOCKFILE="/var/lock/routeadjust"

function dofailover()
{
# Last thing .. we might want to run the failover script ...  If
# there is a lock file, just don't bother (but ignore the lock
# after 2 minutes)
if lockfile -l 120 -1 -r 1 $LOCKFILE.failover ; then
	echo ""
	echo "routeadjust.sh: checking for bandwidth limiting"
	bandlimit-failover.pl 
	rm -f $LOCKFILE.failover
else
	echo
	echo "routeadjust.sh: bandlimit-failover.pl is running (ie. never mind)"
fi
}


# Wait up to 10 minutes for the last instance to exit before we
# try to run
lockfile -l 600 -8 -r 600 $LOCKFILE 
trap "rm -f $LOCKFILE" EXIT

# Don't run two copies - if we are running often, this could
# happen easily ... ip-up triggers adsl-start, triggers route
# triggers blah ... we'll run from cron and fix it all later...

if tty -s ; then TTY=1 ; else  exec >& /var/log/routeadjust.log ; TTY= ; fi

echo "routeadjust.sh $VERSION $0, with parameters $@"
echo ""

pppBASENAME=${0##*/}
pppINTERFACE=$1
pppDEVICE=$2
pppSPEED=$3
pppLOCALIP=$4
pppREMOTEIP=$5
pppIPPARAM=$6

function getptp()
{
	INTERFACE="$1"
        ifconfig |
        awk '(/^'"$INTERFACE"'/) { A=1 }
        (/inet addr/ && A) {
                sub(".*P-t-P:","");
                sub(" .*","");
                print $0;
                exit;
        }'
}

# Wait a sec, just in case the interface is still 'up'
sleep 1

PPP0=`getptp ppp0`
PPP1=`getptp ppp1`

echo "routeadjust.sh: p-t-p ppp0: $PPP0"
echo "routeadjust.sh: p-t-p ppp1: $PPP1"

if [ ! "$PPP1" ] ; then
	# Set up ppp0.mydomain.dyn.ledge.co.za to be the same as
	# ppp0 in the absence of ppp1
	echo "routeadjust.sh: ppp1 absent"
	echo "routeadjust.sh: Setting default route to be via $PPP0"
	route add default gw $PPP0 2>/dev/null
	if [ $( type -p dddns) ] ; then
		echo
		echo "routeadjust.sh: Running dddns -d ppp1 ppp0"
		dddns -d ppp1 ppp0
		echo
		echo "routeadjust.sh: Running dddns"
		dddns
	fi

	dofailover;
	exit
fi

if [ ! "$PPP0" ] ; then
	echo "routeadjust.sh: ppp0 absent"
	dofailover;
	exit
fi

echo "routeadjust.sh: ppp0 and ppp1 simultaneously present"
echo
echo "routeadjust.sh: Setting default route to be via $PPP1 (ppp1)"
# Set our default gateway to be 
route del default gw $PPP0
# route del default gw $PPP1
route add default gw $PPP1 

# ------------------------------------------------------------
# 
# Hacky route via multiple ppp interfaces script -- it's fine,
# but it routes everything via one of the interfaces.
# 

# dev table prio fwmark
echo
echo "routeadjust.sh: setting up outgoing routing via ppp0 and ppp1"
echo "
ppp0   201  201  1
ppp1   202  202  1
" | while read DEV TABLE PRIO FWMARK ; do
	[ $DEV ] || continue
	IP=$( ifconfig $DEV | sed '/inet addr:/ { s/.*inet addr:\([0-9.]*\).*/\1/ ; p } ; d' )
	GW=$( ifconfig $DEV | sed '/P-t-P:/     { s/.*P-t-P:\([0-9.]*\).*/\1/ ;     p } ; d' )

	# % ip rule ls
	# 0:      from all lookup local
	# 123:    from 1.1.1.1 lookup 123
	# 32766:  from all lookup main
	# 32767:  from all lookup default
	ip rule ls |
		grep "^$PRIO:.*from [0-9].*lookup.*$TABLE" |
		cut -f 2 -d " " |
	while read OIP ; do
		if [ "$OIP" = "$IP" ] ; then continue; fi
		echo "  Removing IP rule from $OIP"
		ip rule del prio $PRIO from $OIP table $TABLE
	done
	if ip rule ls | grep "^$PRIO:.*from [0-9].*lookup.*$TABLE" ; then
		echo "  Rule(s) found"
	else
		echo "  Adding IP rule from $IP"
		ip rule add prio $PRIO from $IP table $TABLE
	fi
	ip route flush table $TABLE
	ip route add default via $GW dev $DEV src $IP proto static table $TABLE

#	ip route del $GW dev ppp0  proto static  scope link  src $IP table $TABLE
#	ip route add $GW dev ppp0  proto static  scope link  src $IP table $TABLE
#	ip route del prohibit default table $TABLE metric 1 proto static
#	ip route add prohibit default table $TABLE metric 1 proto static
done

echo
echo "routeadjust.sh: connect to ppp* from eth* rules"
# These rules make sure that you can still connect to your
# outside interface (ppp*) from inside (eth*)
if ip rule ls | grep ^150: >& /dev/null ; then
	echo "  Local route is set up"
else
	echo "  Creating routing table 150 for local eth* interfaces"
	ip route flush table 150
	# Get the existing rules ...
	ip route ls | grep ' dev eth.*scope link' |
	while read NETWORK RULE ; do
		# Duplicate rules are a pain ...
		ip rule del prio 150 from all to $NETWORK lookup 150 >& /dev/null
		ip rule del prio 150 from all to $NETWORK lookup 150 >& /dev/null
		ip rule add prio 150 from all to $NETWORK lookup 150
		# 200.0.0.0/24 dev eth0  proto kernel  scope link  src 200.0.0.5
		ip route add $NETWORK $RULE table 150
	done
fi


# ------------------------------------------------------------

echo
echo "routeadjust.sh: set routes to RSA via gw $PPP0"
# Add routes to south africa via ppp0 - again, just in case
VIA="gw $PPP0"
export VIA
route-to-rsa-via.sh  >& /dev/null

# Now run dddns with our current parameters if it is installed ...
if [ $( type -p dddns) ] ; then
	if [ "$4" ] ; then
		# If we are running from ip-up.d
		echo
		echo "routeadjust.sh: running dddns for $4"
		dddns "$@"
	else
		# If we are running from cron, so set both interfaces
		echo
		echo "routeadjust.sh: running dddns for -d dddns and -d ppp1"
		dddns -d dddns ppp0
		dddns -d ppp1 ppp1
	fi
fi

# Maybe give some feedback about where we're at
if [ $TTY ] ; then 
	echo
	echo "routeadjust.sh: Interfaces"
	ip link | grep ' ppp'
	echo
	echo "# ip rule ls"
	ip rule ls | grep '201\|202\|150'
	echo
	echo "# ip route table 201"
	ip route ls table 201
	echo
	echo "# ip route table 202"
	ip route ls table 202
	echo
	echo "# ip route table 150"
	ip route ls table 150
fi

# And now that we set it all up, check if it's actually okay ...
dofailover

==> dddns-1.1.0.tar.gz: dddns-1.1.0/misc/routeadjust.sh <==
#! /bin/bash

# (c) 2004 Leading Edge Business Solutions and Andrew McGill
# Distribution permitted under terms of GPL licence, as contained
# in the file COPYING
#
VERSION='$Id: routeadjust.sh,v 1.10 2005/03/18 16:16:42 andrewm Exp $' 
VERSION="${VERSION/*,v }"
VERSION="${VERSION/ *}"
#
# Adjust routing for two ppp interfaces.
#
# We route via ppp0 for RSA and ppp1 for world. Additionally we
# route based on source address to avoid asymmetric situations.
# We assume iptables -t nat -A POSTROUTING -j MASQUERADE ... I
# think.  (If not, sorry about the misunderstanding.)
#
# Call this ...
#   * interface comes up - link from /etc/ppp/ip-up.d
#   * interface goes down - link from /etc/ppp/ip-down.d
#   * every so on - to check dddns
#

PATH=/bin:/sbin:/usr/sbin:/usr/local/sbin:/root/bin:/usr/local/bin:/usr/bin
export PATH

LOCKFILE="/var/lock/routeadjust"

function dofailover()
{
# Last thing .. we might want to run the failover script ...  If
# there is a lock file, just don't bother (but ignore the lock
# after 2 minutes)
if lockfile -l 120 -1 -r 1 $LOCKFILE.failover ; then
	echo ""
	echo "routeadjust.sh: checking for bandwidth limiting"
	bandlimit-failover.pl 
	rm -f $LOCKFILE.failover
else
	echo
	echo "routeadjust.sh: bandlimit-failover.pl is running (ie. never mind)"
fi
}


# Wait up to 10 minutes for the last instance to exit before we
# try to run
lockfile -l 600 -8 -r 600 $LOCKFILE 
trap "rm -f $LOCKFILE" EXIT

# Don't run two copies - if we are running often, this could
# happen easily ... ip-up triggers adsl-start, triggers route
# triggers blah ... we'll run from cron and fix it all later...

if tty -s ; then TTY=1 ; else  exec >& /var/log/routeadjust.log ; TTY= ; fi

echo "routeadjust.sh $VERSION $0, with parameters $@"
echo ""

pppBASENAME=${0##*/}
pppINTERFACE=$1
pppDEVICE=$2
pppSPEED=$3
pppLOCALIP=$4
pppREMOTEIP=$5
pppIPPARAM=$6

function getptp()
{
	INTERFACE="$1"
        ifconfig |
        awk '(/^'"$INTERFACE"'/) { A=1 }
        (/inet addr/ && A) {
                sub(".*P-t-P:","");
                sub(" .*","");
                print $0;
                exit;
        }'
}

# Wait a sec, just in case the interface is still 'up'
sleep 1

PPP0=`getptp ppp0`
PPP1=`getptp ppp1`

echo "routeadjust.sh: p-t-p ppp0: $PPP0"
echo "routeadjust.sh: p-t-p ppp1: $PPP1"

if [ ! "$PPP1" ] ; then
	# Set up ppp0.mydomain.dyn.ledge.co.za to be the same as
	# ppp0 in the absence of ppp1
	echo "routeadjust.sh: ppp1 absent"
	echo "routeadjust.sh: Setting default route to be via $PPP0"
	route add default gw $PPP0 2>/dev/null
	if [ $( type -p dddns) ] ; then
		echo "routeadjust.sh: Running dddns"
		dddns
	fi

	dofailover;
	exit
fi

if [ ! "$PPP0" ] ; then
	echo "routeadjust.sh: ppp0 absent"
	dofailover;
	exit
fi

echo "routeadjust.sh: ppp0 and ppp1 simultaneously present"
echo
echo "routeadjust.sh: Setting default route to be via $PPP1 (ppp1)"
# Set our default gateway to be 
route del default gw $PPP0
# route del default gw $PPP1
route add default gw $PPP1 

# ------------------------------------------------------------
# 
# Hacky route via multiple ppp interfaces script -- it's fine,
# but it routes everything via one of the interfaces.
# 

# dev table prio fwmark
echo
echo "routeadjust.sh: setting up outgoing routing via ppp0 and ppp1"
echo "
ppp0   201  201  1
ppp1   202  202  1
" | while read DEV TABLE PRIO FWMARK ; do
	[ $DEV ] || continue
	IP=$( ifconfig $DEV | sed '/inet addr:/ { s/.*inet addr:\([0-9.]*\).*/\1/ ; p } ; d' )
	GW=$( ifconfig $DEV | sed '/P-t-P:/     { s/.*P-t-P:\([0-9.]*\).*/\1/ ;     p } ; d' )

	# % ip rule ls
	# 0:      from all lookup local
	# 123:    from 1.1.1.1 lookup 123
	# 32766:  from all lookup main
	# 32767:  from all lookup default
	ip rule ls |
		grep "^$PRIO:.*from [0-9].*lookup.*$TABLE" |
		cut -f 2 -d " " |
	while read OIP ; do
		if [ "$OIP" = "$IP" ] ; then continue; fi
		echo "  Removing IP rule from $OIP"
		ip rule del prio $PRIO from $OIP table $TABLE
	done
	if ip rule ls | grep "^$PRIO:.*from [0-9].*lookup.*$TABLE" ; then
		echo "  Rule(s) found"
	else
		echo "  Adding IP rule from $IP"
		ip rule add prio $PRIO from $IP table $TABLE
	fi
	ip route flush table $TABLE
	ip route add default via $GW dev $DEV src $IP proto static table $TABLE

#	ip route del $GW dev ppp0  proto static  scope link  src $IP table $TABLE
#	ip route add $GW dev ppp0  proto static  scope link  src $IP table $TABLE
#	ip route del prohibit default table $TABLE metric 1 proto static
#	ip route add prohibit default table $TABLE metric 1 proto static
done

echo
echo "routeadjust.sh: connect to ppp* from eth* rules"
# These rules make sure that you can still connect to your
# outside interface (ppp*) from inside (eth*)
if ip rule ls | grep ^150: >& /dev/null ; then
	echo "  Local route is set up"
else
	echo "  Creating routing table 150 for local eth* interfaces"
	ip route flush table 150
	# Get the existing rules ...
	ip route ls | grep ' dev eth.*scope link' |
	while read NETWORK RULE ; do
		# Duplicate rules are a pain ...
		ip rule del prio 150 from all to $NETWORK lookup 150 >& /dev/null
		ip rule del prio 150 from all to $NETWORK lookup 150 >& /dev/null
		ip rule add prio 150 from all to $NETWORK lookup 150
		# 200.0.0.0/24 dev eth0  proto kernel  scope link  src 200.0.0.5
		ip route add $NETWORK $RULE table 150
	done
fi


# ------------------------------------------------------------

echo
echo "routeadjust.sh: set routes to RSA via gw $PPP0"
# Add routes to south africa via ppp0 - again, just in case
VIA="gw $PPP0"
export VIA
route-to-rsa-via.sh  >& /dev/null

# Now run dddns if it's installed - it can register the links
if [ $( type -p dddns) ] ; then
	echo "routeadjust.sh: running dddns"
	dddns 
fi

# Maybe give some feedback about where we're at
if [ $TTY ] ; then 
	echo
	echo "routeadjust.sh: Interfaces"
	ip link | grep ' ppp'
	echo
	echo "# ip rule ls"
	ip rule ls | grep '201\|202\|150'
	echo
	echo "# ip route table 201"
	ip route ls table 201
	echo
	echo "# ip route table 202"
	ip route ls table 202
	echo
	echo "# ip route table 150"
	ip route ls table 150
fi

# And now that we set it all up, check if it's actually okay ...
dofailover
